@@ -4,9 +4,9 @@
 export WORKSPACE=/home/jenkins/workspace
 
 export OS_BUILD=64
-export PG_VER=12
+export PG_VER=13
 export PGPATH=${WORKSPACE}/pg/label/${label}/rel/pg${PG_VER}w${OS_BUILD}
-export PATH=${PGPATH}/bin:${PGPATH}/lib:${PATH}
+export PATH=${PATH}:${PGPATH}/bin:${PGPATH}/lib
 export PGPORT=55432
 export PGDATA=$PGPATH/data_${PGPORT}
 export PGLOG="$PGDATA/pgsql.log"
