@@ -1267,13 +1267,13 @@ gserialized_joinsel_internal(PlannerInfo *root, List *args, JoinType jointype, i
 	Var *var1 = (Var*) arg1;
 	Var *var2 = (Var*) arg2;
 
-	elog(DEBUG2, "%s: entered function", __func__);
+	POSTGIS_DEBUGF(2, "%s: entered function", __func__);
 
 	/* We only do column joins right now, no functional joins */
 	/* TODO: handle g1 && ST_Expand(g2) */
 	if (!IsA(arg1, Var) || !IsA(arg2, Var))
 	{
-		elog(DEBUG1, "%s called with arguments that are not column references", __func__);
+		POSTGIS_DEBUGF(1, "%s called with arguments that are not column references", __func__);
 		return DEFAULT_ND_JOINSEL;
 	}
 
@@ -1288,17 +1288,17 @@ gserialized_joinsel_internal(PlannerInfo *root, List *args, JoinType jointype, i
 	/* If we can't get stats, we have to stop here! */
 	if (!stats1)
 	{
-		elog(DEBUG2, "%s: cannot find stats for \"%s\"",  __func__, get_rel_name(relid2) ? get_rel_name(relid2) : "NULL");
+		POSTGIS_DEBUGF(2, "%s: cannot find stats for \"%s\"",  __func__, get_rel_name(relid2) ? get_rel_name(relid2) : "NULL");
 		return DEFAULT_ND_JOINSEL;
 	}
 	else if (!stats2)
 	{
-		elog(DEBUG2, "%s: cannot find stats for \"%s\"",  __func__, get_rel_name(relid2) ? get_rel_name(relid2) : "NULL");
+		POSTGIS_DEBUGF(2, "%s: cannot find stats for \"%s\"",  __func__, get_rel_name(relid2) ? get_rel_name(relid2) : "NULL");
 		return DEFAULT_ND_JOINSEL;
 	}
 
 	selectivity = estimate_join_selectivity(stats1, stats2);
-	elog(DEBUG2, "got selectivity %g", selectivity);
+	POSTGIS_DEBUGF(2, "got selectivity %g", selectivity);
 	pfree(stats1);
 	pfree(stats2);
 	return selectivity;
@@ -1321,19 +1321,19 @@ Datum gserialized_gist_joinsel(PG_FUNCTION_ARGS)
 	JoinType jointype = (JoinType) PG_GETARG_INT16(3);
 	int mode = PG_GETARG_INT32(4);
 
-	elog(DEBUG2, "%s: entered function", __func__);
+	POSTGIS_DEBUGF(2, "%s: entered function", __func__);
 
 	/* Check length of args and punt on > 2 */
 	if (list_length(args) != 2)
 	{
-		elog(DEBUG2, "%s: got nargs == %d", __func__, list_length(args));
+		POSTGIS_DEBUGF(2, "%s: got nargs == %d", __func__, list_length(args));
 		PG_RETURN_FLOAT8(DEFAULT_ND_JOINSEL);
 	}
 
 	/* Only respond to an inner join/unknown context join */
 	if (jointype != JOIN_INNER)
 	{
-		elog(DEBUG1, "%s: jointype %d not supported", __func__, jointype);
+		POSTGIS_DEBUGF(1, "%s: jointype %d not supported", __func__, jointype);
 		PG_RETURN_FLOAT8(DEFAULT_ND_JOINSEL);
 	}
 
@@ -2235,31 +2235,31 @@ gserialized_sel_internal(PlannerInfo *root, List *args, int varRelid, int mode)
 	GBOX search_box;
 	float8 selectivity = 0;
 
-	elog(DEBUG2, "%s: entered function", __func__);
+	POSTGIS_DEBUGF(2, "%s: entered function", __func__);
 
 	if (!get_restriction_variable(root, args, varRelid, &vardata, &other, &varonleft))
 	{
-		elog(DEBUG2, "%s: could not find vardata", __func__);
+		POSTGIS_DEBUGF(2, "%s: could not find vardata", __func__);
 		return DEFAULT_ND_SEL;
 	}
 
 	if (!IsA(other, Const))
 	{
 		ReleaseVariableStats(vardata);
-		elog(DEBUG2, "%s: no constant argument, returning default selectivity %g", __func__, DEFAULT_ND_SEL);
+		POSTGIS_DEBUGF(2, "%s: no constant argument, returning default selectivity %g", __func__, DEFAULT_ND_SEL);
 		return DEFAULT_ND_SEL;
 	}
 
 	if (!gserialized_datum_get_gbox_p(((Const*)other)->constvalue, &search_box))
 	{
 		ReleaseVariableStats(vardata);
-		elog(DEBUG2, "%s: search box is EMPTY", __func__);
+		POSTGIS_DEBUGF(2, "%s: search box is EMPTY", __func__);
 		return 0.0;
 	}
 
 	if (!vardata.statsTuple)
 	{
-		elog(DEBUG1, "%s: no statistics available on table. Empty? Need to ANALYZE?", __func__);
+		POSTGIS_DEBUGF(1, "%s: no statistics available on table. Empty? Need to ANALYZE?", __func__);
 		return DEFAULT_ND_SEL;
 	}
 
@@ -2279,7 +2279,7 @@ Datum gserialized_gist_sel(PG_FUNCTION_ARGS)
 	int varRelid = PG_GETARG_INT32(3);
 	int mode = PG_GETARG_INT32(4);
 	float8 selectivity = gserialized_sel_internal(root, args, varRelid, mode);
-	elog(DEBUG2, "%s: selectivity is %g", __func__, selectivity);
+	POSTGIS_DEBUGF(2, "%s: selectivity is %g", __func__, selectivity);
 	PG_RETURN_FLOAT8(selectivity);
 }
 
@@ -2303,6 +2303,9 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 	bool only_parent = false;
 	int key_type;
 
+	/* We need to initialize the internal cache to access it later via postgis_oid() */
+	postgis_initialize_cache(fcinfo);
+
 	if ( PG_NARGS() == 4 )
 	{
 		nsp = text_to_cstring(PG_GETARG_TEXT_P(0));
@@ -2343,7 +2346,7 @@ Datum gserialized_estimated_extent(PG_FUNCTION_ARGS)
 	/* Read the extent from the head of the spatial index, if there is one */
 	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type);
 	if (!idx_oid)
-		elog(DEBUG2, "index for \"%s.%s\" does not exist", tbl, text_to_cstring(col));
+		POSTGIS_DEBUGF(2, "index for \"%s.%s\" does not exist", tbl, text_to_cstring(col));
 	gbox = spatial_index_read_extent(idx_oid, key_type);
 #endif
 
@@ -2565,8 +2568,12 @@ Datum _postgis_gserialized_index_extent(PG_FUNCTION_ARGS)
 	int key_type;
 	Oid tbl_oid = PG_GETARG_DATUM(0);
 	text *col = PG_GETARG_TEXT_P(1);
+	Oid idx_oid;
 
-	Oid idx_oid = table_get_spatial_index(tbl_oid, col, &key_type);
+	/* We need to initialize the internal cache to access it later via postgis_oid() */
+	postgis_initialize_cache(fcinfo);
+
+	idx_oid = table_get_spatial_index(tbl_oid, col, &key_type);
 	if (!idx_oid)
 		PG_RETURN_NULL();
 
