@@ -794,9 +794,8 @@ Datum LWGEOM_from_WKB(PG_FUNCTION_ARGS)
 	uint8_t *wkb = (uint8_t*)VARDATA(bytea_wkb);
 
 	lwgeom = lwgeom_from_wkb(wkb, VARSIZE_ANY_EXHDR(bytea_wkb), LW_PARSER_CHECK_ALL);
-
-	if ( lwgeom_needs_bbox(lwgeom) )
-		lwgeom_add_bbox(lwgeom);
+	if (!lwgeom)
+		lwpgerror("Unable to parse WKB");
 
 	geom = geometry_serialize(lwgeom);
 	lwgeom_free(lwgeom);
