@@ -1,4 +1,6 @@
 
+#include "../postgis_config.h"
+
 /* PostgreSQL headers */
 #include "postgres.h"
 #include "funcapi.h"
@@ -17,12 +19,15 @@
 #include "utils/datetime.h"
 #include "utils/lsyscache.h"
 #include "utils/json.h"
+#if POSTGIS_PGSQL_VERSION < 130
 #include "utils/jsonapi.h"
+#else
+#include "common/jsonapi.h"
+#endif
 #include "utils/typcache.h"
 #include "utils/syscache.h"
 
 /* PostGIS headers */
-#include "../postgis_config.h"
 #include "lwgeom_pg.h"
 #include "lwgeom_log.h"
 #include "liblwgeom.h"
@@ -81,8 +86,13 @@ ST_AsGeoJsonRow(PG_FUNCTION_ARGS)
 	bool		do_pretty = PG_GETARG_BOOL(3);
 	StringInfo	result;
 	char        *geom_column = text_to_cstring(geom_column_text);
-	Oid         geom_oid = postgis_oid_fcinfo(fcinfo, GEOMETRYOID);
-	Oid         geog_oid = postgis_oid_fcinfo(fcinfo, GEOGRAPHYOID);
+	Oid geom_oid = InvalidOid;
+	Oid geog_oid = InvalidOid;
+
+	/* We need to initialize the internal cache to access it later via postgis_oid() */
+	postgis_initialize_cache(fcinfo);
+	geom_oid = postgis_oid(GEOMETRYOID);
+	geog_oid = postgis_oid(GEOGRAPHYOID);
 
 	if (strlen(geom_column) == 0)
 		geom_column = NULL;
